package com.exner.tools.immichuploader.preferences

import androidx.datastore.core.DataStore
import androidx.datastore.preferences.core.Preferences
import androidx.datastore.preferences.core.booleanPreferencesKey
import androidx.datastore.preferences.core.edit
import androidx.datastore.preferences.core.emptyPreferences
import androidx.datastore.preferences.core.stringPreferencesKey
import androidx.datastore.preferences.core.stringSetPreferencesKey
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.catch
import kotlinx.coroutines.flow.map
import javax.inject.Inject

class UserPreferencesRepositoryImpl @Inject constructor(
    private val userDataStorePreferences: DataStore<Preferences>
) : UserPreferencesRepository {

    override fun immichServerUrl(): Flow<String?> {
        return userDataStorePreferences.data.catch {
            emit(emptyPreferences())
        }.map { preferences ->
            preferences[KEY_IMMICH_SERVER_URL]
        }
    }

    override suspend fun setImmichServerUrl(newUrl: String?) {
        if (newUrl.isNullOrEmpty()) {
            userDataStorePreferences.edit { preferences ->
                preferences.remove(KEY_IMMICH_SERVER_URL)
            }
        } else {
            userDataStorePreferences.edit { preferences ->
                preferences[KEY_IMMICH_SERVER_URL] = newUrl
            }
        }
    }

    override fun immichApiKey(): Flow<String?> {
        return userDataStorePreferences.data.catch {
            emit(emptyPreferences())
        }.map { preferences ->
            preferences[KEY_IMMICH_API_KEY]
        }
    }

    override suspend fun setImmichApiKey(newKey: String?) {
        if (newKey.isNullOrEmpty()) {
            userDataStorePreferences.edit { preferences ->
                preferences.remove(KEY_IMMICH_API_KEY)
            }
        } else {
            userDataStorePreferences.edit { preferences ->
                preferences[KEY_IMMICH_API_KEY] = newKey
            }
        }
    }

    override fun availablePseudoFolders(): Flow<Set<String>> {
        return userDataStorePreferences.data.catch {
            emit(emptyPreferences())
        }.map { preferences ->
            preferences[KEY_AVAILABLE_PSEUDO_FOLDERS] ?: emptySet()
        }
    }

    override suspend fun setAvailablePseudoFolders(folders: Set<String>) {
        userDataStorePreferences.edit { preferences ->
            preferences[KEY_AVAILABLE_PSEUDO_FOLDERS] = folders
        }
    }

    override fun deleteMediaAfterSuccessfulUploadPerPseudoFolder(): Flow<Set<String>> {
        return userDataStorePreferences.data.catch {
            emit(emptyPreferences())
        }.map { preferences ->
            preferences[KEY_DELETE_MEDIA_AFTER_UPLOAD_PER_PSEUDO_FOLDER_SERIALIZED] ?: setOf(
                "Download",
                "Pictures/Screenshots",
                "Pictures/Threema",
                "Movies/Threema"
            )
        }
    }

    override suspend fun setDeleteMediaAfterSuccessfulUploadPerPseudoFolder(deleteAfterUploadPPF: Set<String>) {
        userDataStorePreferences.edit { preferences ->
            preferences[KEY_DELETE_MEDIA_AFTER_UPLOAD_PER_PSEUDO_FOLDER_SERIALIZED] =
                deleteAfterUploadPPF
        }
    }

    override fun latestMediaStoreVersion(): Flow<String> {
        return userDataStorePreferences.data.catch {
            emit(emptyPreferences())
        }.map { preferences ->
            preferences[KEY_LATEST_MEDIA_STORE_VERSION] ?: ""
        }
    }

    override suspend fun setLatestMediaStoreVersion(version: String) {
        userDataStorePreferences.edit { preferences ->
            preferences[KEY_LATEST_MEDIA_STORE_VERSION] = version
        }
    }

    override fun approveEachDeletion(): Flow<Boolean> {
        return userDataStorePreferences.data.catch {
            emit(emptyPreferences())
        }.map { preferences ->
            preferences[KEY_APPROVE_EACH_DELETION] == true
        }
    }

    override suspend fun setApproveEachDeletion(approve: Boolean) {
        userDataStorePreferences.edit { preferences ->
            preferences[KEY_APPROVE_EACH_DELETION] = approve
        }
    }

    private companion object {
        val KEY_IMMICH_SERVER_URL = stringPreferencesKey(name = "preference_immich_server_url")
        val KEY_IMMICH_API_KEY = stringPreferencesKey(name = "preference_immich_api_key")
        val KEY_DELETE_MEDIA_AFTER_UPLOAD_PER_PSEUDO_FOLDER_SERIALIZED =
            stringSetPreferencesKey(name = "preference_delete_media_after_upload_per_pseudo_folder_serialized")
        val KEY_LATEST_MEDIA_STORE_VERSION =
            stringPreferencesKey(name = "latest_media_store_version")
        val KEY_AVAILABLE_PSEUDO_FOLDERS =
            stringSetPreferencesKey(name = "preference_available_pseudo_filters")
        val KEY_APPROVE_EACH_DELETION = booleanPreferencesKey("preference_approve_each_deletion")
    }
}