package com.exner.tools.immichuploader.ui

import android.app.Activity.RESULT_OK
import android.widget.Toast
import androidx.activity.compose.rememberLauncherForActivityResult
import androidx.activity.result.ActivityResult
import androidx.activity.result.contract.ActivityResultContracts
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.imePadding
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.Delete
import androidx.compose.material.icons.filled.MoreVert
import androidx.compose.material3.BottomAppBar
import androidx.compose.material3.Button
import androidx.compose.material3.DropdownMenu
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExtendedFloatingActionButton
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.ListItem
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Switch
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import coil.ImageLoader
import coil.compose.AsyncImage
import coil.decode.VideoFrameDecoder
import com.ramcosta.composedestinations.annotation.Destination
import com.ramcosta.composedestinations.annotation.RootGraph
import com.ramcosta.composedestinations.navigation.DestinationsNavigator

@OptIn(ExperimentalMaterial3Api::class)
@Destination<RootGraph>
@Composable
fun DeleteScreen(
    deleteScreenViewModel: DeleteScreenViewModel = hiltViewModel(),
    destinationsNavigator: DestinationsNavigator
) {
    val context = LocalContext.current

    val filesForDeletion by deleteScreenViewModel.filesToDelete.collectAsStateWithLifecycle(
        emptyList()
    )
    val numberOfFilesToDelete by deleteScreenViewModel.countFilesToDelete.collectAsStateWithLifecycle(
        0
    )

    var deleteButtonActive by remember { mutableStateOf(false) }
    val launcher = rememberLauncherForActivityResult(
        contract = ActivityResultContracts.StartIntentSenderForResult(),
        onResult = { result: ActivityResult? ->
            if (result!!.resultCode == RESULT_OK) {
                Toast.makeText(context, "File(s) deleted.", Toast.LENGTH_SHORT).show()
            }
        })
    LaunchedEffect(launcher) {
        deleteButtonActive = true
    }

    var displayMainMenu by remember { mutableStateOf(false) }

    Scaffold(
        topBar = {
            TopAppBar(
                title = {
                    Text(text = "Immich Uploader")
                },
                navigationIcon = {},
                actions = {
                    IconButton(
                        onClick = {
                            displayMainMenu = !displayMainMenu
                        }
                    ) {
                        Icon(
                            imageVector = Icons.Filled.MoreVert,
                            contentDescription = "Menu"
                        )
                    }
                    DropdownMenu(
                        expanded = displayMainMenu,
                        onDismissRequest = { displayMainMenu = false }
                    ) {
//                        DropdownMenuItem(
//                            enabled = destination != AboutDestination,
//                            text = { Text(text = "About Meditation Timer") },
//                            onClick = {
//                                displayMainMenu = false
//                                destinationsNavigator.navigate(AboutDestination)
//                            }
//                        )
                    }
                }
            )
        },
        content = { innerPadding ->
            Column(
                modifier = Modifier
                    .imePadding()
                    .padding(innerPadding)
                    .padding(8.dp)
                    .fillMaxSize()
            ) {
                val imageLoader = ImageLoader.Builder(LocalContext.current)
                    .components {
                        add(VideoFrameDecoder.Factory())
                    }
                    .build()

                Text(text = "$numberOfFilesToDelete files marked for deletion.")
                Spacer(modifier = Modifier.size(8.dp))
                LazyColumn(
                    modifier = Modifier
                        .fillMaxWidth()
                        .weight(0.7f)
                ) {
                    items(items = filesForDeletion, key = { it.uid }) { fileToDelete ->
                        ListItem(
                            headlineContent = {
                                Text(text = fileToDelete.fileName)
                            },
                            leadingContent = {
                                AsyncImage(
                                    model = fileToDelete.uriString,
                                    imageLoader = imageLoader,
                                    contentDescription = fileToDelete.fileName,
                                    contentScale = ContentScale.Crop,
                                    modifier = Modifier.size(50.dp)
                                )
                            },
                            trailingContent = {
                                Switch(
                                    checked = fileToDelete.delete,
                                    onCheckedChange = {
                                        deleteScreenViewModel.markFileForDeletionInRepository(
                                            fileToDelete,
                                            delete = it
                                        )
                                    }
                                )
                            }
                        )
                    }
                }
                Spacer(modifier = Modifier.size(8.dp))
                Row(
                    modifier = Modifier.fillMaxWidth(),
                    horizontalArrangement = Arrangement.SpaceBetween
                ) {
                    Button(onClick = {
                        deleteScreenViewModel.markAllFilesForDeletionInRepository(true)
                    }) {
                        Text(text = "Select all")
                    }
                    Button(onClick = {
                        deleteScreenViewModel.markAllFilesForDeletionInRepository(false)
                    }) {
                        Text(text = "Clear selection")
                    }
                }
                Spacer(modifier = Modifier.size(8.dp))
                Text(text = "Uncheck assets if you do not want them deleted.")
            }
        },
        bottomBar = {
            BottomAppBar(
                actions = {
                    IconButton(
                        onClick = {
                            destinationsNavigator.navigateUp()
                        }
                    ) {
                        Icon(
                            imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                            contentDescription = "Back"
                        )
                    }
                },
                floatingActionButton = {
                    if (deleteButtonActive) {
                        ExtendedFloatingActionButton(onClick = {
                            deleteScreenViewModel.deleteQueuedAssets(
                                launcher = launcher,
                                context = context,
                                callback = {
                                    destinationsNavigator.navigateUp()
                                }
                            )
                        }) {
                            Icon(
                                imageVector = Icons.Default.Delete,
                                contentDescription = "Delete"
                            )
                            Text(text = "Delete")
                        }
                    }
                }
            )
        }
    )
}
