package com.exner.tools.immichuploader.ui

import android.app.Application
import android.content.Context
import android.os.Build
import android.util.Log
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.IntentSenderRequest
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.exner.tools.immichuploader.data.MyRepository
import com.exner.tools.immichuploader.data.entities.FileDeletionCache
import com.exner.tools.immichuploader.preferences.UserPreferencesRepository
import com.exner.tools.immichuploader.work.deleteImage
import com.exner.tools.immichuploader.work.deleteImagesAboveAPI30
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.firstOrNull
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class DeleteScreenViewModel @Inject constructor(
    application: Application,
    val userPreferencesRepository: UserPreferencesRepository,
    private val repository: MyRepository,
) : ViewModel() {

    val countFilesToDelete = repository.countFilesToDelete
    val filesToDelete = repository.observeFilesToDelete

    fun markFileForDeletionInRepository(fileToDelete: FileDeletionCache, delete: Boolean) {
        viewModelScope.launch {
            val newFileToDelete = fileToDelete.copy(
                delete = delete
            )
            repository.updateFileToDelete(newFileToDelete)
        }
    }

    fun markAllFilesForDeletionInRepository(delete: Boolean) {
        viewModelScope.launch {
            repository.setDeleteFlagForAllInDeletionCache(delete)
        }
    }

    fun deleteQueuedAssets(
        launcher: ActivityResultLauncher<IntentSenderRequest>,
        context: Context,
        callback: () -> Unit
    ) {
        viewModelScope.launch {
            val allFilesToDelete = repository.getAllFilesToDelete().filter {
                it.delete == true
            }
            val approveEachDeletion =
                userPreferencesRepository.approveEachDeletion().firstOrNull() == true

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R && !approveEachDeletion) {
                Log.d("DSVM", "Deleting images (API30+)")
                // on R/30/Android 11 and above, this will pop up a question "delete these 7 files?"
                deleteImagesAboveAPI30(launcher, context, repository, allFilesToDelete.toList())
            } else {
                Log.d("DSVM", "Deleting images (<API30)")
                // below R/30, there will be a popup for each file
                allFilesToDelete.forEach { fileToDelete ->
                    deleteImage(launcher, context, repository, fileToDelete)
                }
            }
            // done
            callback()
        }
    }
}
