package com.exner.tools.immichuploader.ui

import android.util.Log
import androidx.compose.animation.AnimatedVisibility
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.imePadding
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.wrapContentHeight
import androidx.compose.foundation.lazy.staggeredgrid.LazyVerticalStaggeredGrid
import androidx.compose.foundation.lazy.staggeredgrid.StaggeredGridCells
import androidx.compose.foundation.lazy.staggeredgrid.items
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.AccountCircle
import androidx.compose.material.icons.filled.Close
import androidx.compose.material.icons.filled.Delete
import androidx.compose.material.icons.filled.MoreVert
import androidx.compose.material.icons.filled.PlayArrow
import androidx.compose.material.icons.filled.Refresh
import androidx.compose.material.icons.filled.Settings
import androidx.compose.material3.BottomAppBar
import androidx.compose.material3.DropdownMenu
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExtendedFloatingActionButton
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.LinearProgressIndicator
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Switch
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.Observer
import androidx.lifecycle.compose.LocalLifecycleOwner
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import androidx.work.WorkInfo
import coil.ImageLoader
import coil.compose.AsyncImage
import coil.decode.VideoFrameDecoder
import com.exner.tools.immichuploader.Permissions
import com.exner.tools.immichuploader.work.UploadMediaAssetWorker
import com.google.accompanist.permissions.ExperimentalPermissionsApi
import com.google.accompanist.permissions.rememberMultiplePermissionsState
import com.ramcosta.composedestinations.annotation.Destination
import com.ramcosta.composedestinations.annotation.RootGraph
import com.ramcosta.composedestinations.generated.destinations.DeleteScreenDestination
import com.ramcosta.composedestinations.generated.destinations.SettingsDestination
import com.ramcosta.composedestinations.navigation.DestinationsNavigator

@OptIn(ExperimentalPermissionsApi::class, ExperimentalMaterial3Api::class)
@Destination<RootGraph>(start = true)
@Composable
fun HomeScreen(
    homeScreenViewModel: HomeScreenViewModel = hiltViewModel(),
    destinationsNavigator: DestinationsNavigator
) {
    val context = LocalContext.current

    val processState by homeScreenViewModel.processStateFlow.collectAsStateWithLifecycle()

    val mediaFiles by homeScreenViewModel.mediaFiles.collectAsStateWithLifecycle(
        emptyList()
    )
    val mediaFilesNotBackedUp by homeScreenViewModel.mediaFilesNotBackedUp.collectAsStateWithLifecycle(
        emptyList()
    )
    val numberOfMediaFilesInDatabase by homeScreenViewModel.countMediaFiles.collectAsStateWithLifecycle(
        0
    )
    val numberOfMediaFilesInDatabaseNotBackedUp by homeScreenViewModel.countMediaFilesNotBackedUp.collectAsStateWithLifecycle(
        0
    )
    val numberOfFilesToDelete by homeScreenViewModel.countFilesToDelete.collectAsStateWithLifecycle(
        0
    )

    val uploadProgress: Float by homeScreenViewModel.uploadProgress.collectAsStateWithLifecycle(
        initialValue = 0.0f
    )

    var uploadedAsset: String? = remember { null }

    val permissions = Permissions(context = context)
    var weHaveAllPermissions by remember { mutableStateOf(permissions.doWeHaveAllPermissions()) }
    val permissionsNeeded = rememberMultiplePermissionsState(
        permissions = permissions.getAllNecessaryPermissionsAsListOfStrings(),
        onPermissionsResult = {
            permissions.checkNecessaryPermissions()
            weHaveAllPermissions = permissions.doWeHaveAllPermissions()
        }
    )

    val currentLifeCycleOwner = LocalLifecycleOwner.current

    val discoveryObserver: Observer<WorkInfo?> = Observer {
        if (it?.state?.isFinished == true) {
            val result = it.outputData
            Log.d("HomeScreen", "D Worker result $result")
            if (it.state == WorkInfo.State.FAILED) {
                Log.d("HomeScreen", "D Worker work failed ${it.stopReason}")
            }
            homeScreenViewModel.updateProcessState(ProcessStateConstants.IDLE)
        } else if (it?.state == WorkInfo.State.ENQUEUED) {
            Log.d("HomeScreen", "D Worker enqueued ${it.state}")
            homeScreenViewModel.updateProcessState(ProcessStateConstants.DISCOVERING)
        } else if (it?.state == WorkInfo.State.CANCELLED) {
            Log.d("HomeScreen", "D Worker cancelled ${it.state}")
            homeScreenViewModel.updateProcessState(ProcessStateConstants.CANCELLING)
        } else {
            Log.d("HomeScreen", "D Worker ${it?.progress} - ${it?.state}")
        }
    }

    val uploadObserver: Observer<WorkInfo?> = Observer { workInfo ->
        if (workInfo?.state?.isFinished == true) {
            val result = workInfo.outputData
            Log.d("HomeScreen", "U Worker result $result")
            if (workInfo.state == WorkInfo.State.FAILED) {
                Log.d("HomeScreen", "U Worker work failed ${workInfo.stopReason}")
            }
            homeScreenViewModel.updateProcessState(ProcessStateConstants.IDLE)
        } else if (workInfo?.state == WorkInfo.State.ENQUEUED) {
            Log.d("HomeScreen", "U Worker enqueued ${workInfo.state}")
            homeScreenViewModel.updateProcessState(ProcessStateConstants.UPLOADING)
        } else if (workInfo?.state == WorkInfo.State.CANCELLED) {
            Log.d("HomeScreen", "U Worker cancelled ${workInfo.state}")
            homeScreenViewModel.updateProcessState(ProcessStateConstants.CANCELLING)
        } else {
            Log.d("HomeScreen", "U Worker state ${workInfo?.progress} - ${workInfo?.state}")
            val progress = workInfo?.progress?.getFloat(UploadMediaAssetWorker.PROGRESS, 0.0f)
            if (progress != null) {
                homeScreenViewModel.setUploadProgress(progress)
                val currentAsset = workInfo.progress.getString(UploadMediaAssetWorker.CURRENT_ASSET)
                val status = workInfo.progress.getString(UploadMediaAssetWorker.STATUS)
                if (currentAsset != null && status != null) {
                    uploadedAsset = currentAsset
                }
            }
        }
    }

    // trigger discovery of media. ViewModel will decide if that is actually necessary
    homeScreenViewModel.updateMedia(
        context = context,
        lifecycleOwner = currentLifeCycleOwner,
        observer = discoveryObserver
    )

    var showOnlyNew by remember { mutableStateOf(true) }
    var displayMainMenu by remember { mutableStateOf(false) }

    Scaffold(
        topBar = {
            TopAppBar(
                title = {
                    Text(text = "Immich Uploader")
                },
                navigationIcon = {},
                actions = {
                    IconButton(
                        onClick = {
                            displayMainMenu = !displayMainMenu
                        }
                    ) {
                        Icon(
                            imageVector = Icons.Filled.MoreVert,
                            contentDescription = "Menu"
                        )
                    }
                    DropdownMenu(
                        expanded = displayMainMenu,
                        onDismissRequest = { displayMainMenu = false }
                    ) {
                        DropdownMenuItem(
                            enabled = true,
                            text = { Text(text = "Export settings") },
                            onClick = {
                                displayMainMenu = false
                                homeScreenViewModel.exportSettings(context)
                            }
                        )
//                        HorizontalDivider()
//                        DropdownMenuItem(
//                            enabled = destination != AboutDestination,
//                            text = { Text(text = "About Meditation Timer") },
//                            onClick = {
//                                displayMainMenu = false
//                                destinationsNavigator.navigate(AboutDestination)
//                            }
//                        )
                    }
                }
            )
        },
        content = { innerPadding ->
            Column(
                modifier = Modifier
                    .imePadding()
                    .padding(innerPadding)
                    .padding(8.dp)
                    .fillMaxSize()
            ) {
                Text(
                    text = "$numberOfMediaFilesInDatabase file" + if (numberOfMediaFilesInDatabase == 1) {
                        ""
                    } else {
                        "s"
                    } + ", $numberOfMediaFilesInDatabaseNotBackedUp not yet backed up",
                    modifier = Modifier.padding(0.dp, 4.dp)
                )
                val percentageDisplayString: String = if (uploadProgress < 0.01) {
                    "%.2f".format(uploadProgress * 100f)
                } else if (uploadProgress < 0.1) {
                    "%.1f".format(uploadProgress * 100f)
                } else {
                    "%.0f".format(uploadProgress * 100f)
                }
                Text(
                    text = "Status: ${processState.currentState}" + if (processState.currentState == ProcessStateConstants.UPLOADING) {
                        " - ${percentageDisplayString}%"
                    } else {
                        ""
                    },
                    modifier = Modifier.padding(0.dp, 4.dp)
                )
                AnimatedVisibility(
                    visible = processState.currentState == ProcessStateConstants.PROBLEM
                ) {
                    Text(
                        text = "Problem: ${processState.message}",
                        modifier = Modifier.padding(0.dp, 4.dp)
                    )
                }
                AnimatedVisibility(
                    visible = processState.currentState == ProcessStateConstants.DISCOVERING
                ) {
                    LinearProgressIndicator(
                        modifier = Modifier.fillMaxWidth(),
                        color = MaterialTheme.colorScheme.secondary,
                        trackColor = MaterialTheme.colorScheme.surfaceVariant,
                    )
                }
                AnimatedVisibility(
                    visible = processState.currentState != ProcessStateConstants.DISCOVERING && numberOfFilesToDelete > 0
                ) {
                    Text(
                        text = "$numberOfFilesToDelete file" + if (numberOfFilesToDelete == 1) {
                            ""
                        } else {
                            "s"
                        } + " marked for deletion",
                        modifier = Modifier.padding(0.dp, 4.dp)
                    )
                }
                AnimatedVisibility(
                    visible = processState.currentState == ProcessStateConstants.IDLE
                ) {
                    Text(
                        text = "Tap an image or video to upload it.",
                        modifier = Modifier.padding(0.dp, 4.dp)
                    )
                }
                AnimatedVisibility(
                    visible = processState.currentState == ProcessStateConstants.UPLOADING
                ) {
                    LinearProgressIndicator(
                        progress = { uploadProgress },
                        modifier = Modifier.fillMaxWidth(),
                        color = MaterialTheme.colorScheme.secondary,
                        trackColor = MaterialTheme.colorScheme.surfaceVariant,
                    )
                }

                AnimatedVisibility(
                    visible = processState.currentState != ProcessStateConstants.UPLOADING
                ) {
                    // list of media files
                    Row(
                        modifier = Modifier
                            .fillMaxWidth()
                            .background(MaterialTheme.colorScheme.primaryContainer),
                        horizontalArrangement = Arrangement.SpaceBetween,
                        verticalAlignment = Alignment.CenterVertically
                    ) {
                        Text(
                            text = "Media",
                            color = MaterialTheme.colorScheme.onPrimaryContainer
                        )

                        Text(text = "New files only")

                        Switch(
                            checked = showOnlyNew,
                            onCheckedChange = {
                                showOnlyNew = it
                            }
                        )
                    }
                }

                val mediaFilesToShow = if (showOnlyNew) {
                    mediaFilesNotBackedUp
                } else {
                    mediaFiles
                }

                AnimatedVisibility(
                    visible = mediaFilesToShow.isEmpty() && processState.currentState != ProcessStateConstants.UPLOADING
                ) {
                    Box(
                        modifier = Modifier.fillMaxSize()
                    ) {
                        Text(
                            text = "Press refresh at the bottom to look for media files",
                            modifier = Modifier.align(Alignment.Center)
                        )
                    }
                }
                AnimatedVisibility(
                    visible = mediaFilesToShow.isNotEmpty() && processState.currentState != ProcessStateConstants.UPLOADING
                ) {
                    val imageLoader = ImageLoader.Builder(LocalContext.current)
                        .components {
                            add(VideoFrameDecoder.Factory())
                        }
                        .build()

                    LazyVerticalStaggeredGrid(
                        columns = StaggeredGridCells.Fixed(2),
                        verticalItemSpacing = 4.dp,
                        horizontalArrangement = Arrangement.spacedBy(4.dp),
                        modifier = Modifier.padding(0.dp, 8.dp)
                    ) {
                        items(items = mediaFilesToShow, key = { it.uri }) { mediaFile ->
                            AsyncImage(
                                model = mediaFile.uri,
                                imageLoader = imageLoader,
                                contentDescription = mediaFile.uri,
                                contentScale = ContentScale.Crop,
                                modifier = Modifier
                                    .fillMaxWidth()
                                    .wrapContentHeight()
                                    .clickable(
                                        onClick = {
                                            homeScreenViewModel.backupSingleAssetSafely(
                                                context = context,
                                                mediaFile = mediaFile
                                            )
                                        }
                                    )
                            )
                        }
                    }
                }
                if (processState.currentState == ProcessStateConstants.UPLOADING) {
                    if (uploadedAsset != null) {
                        Spacer(modifier = Modifier.size(8.dp))
                        Text(text = uploadedAsset.toString())
                    }
                }
            }
        },
        bottomBar = {
            BottomAppBar(
                actions = {
                    AnimatedVisibility(
                        visible = weHaveAllPermissions
                    ) {
                        IconButton(
                            onClick = {
                                homeScreenViewModel.updateMedia(
                                    context = context,
                                    lifecycleOwner = currentLifeCycleOwner,
                                    observer = discoveryObserver,
                                    force = true
                                )
                            },
                            enabled = processState.currentState == ProcessStateConstants.IDLE || processState.currentState == ProcessStateConstants.PROBLEM
                        ) {
                            Icon(
                                imageVector = Icons.Default.Refresh,
                                contentDescription = "Refresh Images"
                            )
                        }
                    }
                    IconButton(
                        onClick = {
                            destinationsNavigator.navigate(SettingsDestination)
                        },
                        enabled = processState.currentState == ProcessStateConstants.IDLE || processState.currentState == ProcessStateConstants.PROBLEM
                    ) {
                        Icon(
                            imageVector = Icons.Default.Settings,
                            contentDescription = "Settings"
                        )
                    }
                    if (numberOfFilesToDelete > 0) {
                        IconButton(
                            onClick = {
                                destinationsNavigator.navigate(DeleteScreenDestination)
                            },
                            enabled = processState.currentState == ProcessStateConstants.IDLE
                        ) {
                            Icon(
                                imageVector = Icons.Default.Delete,
                                contentDescription = "Delete uploaded assets"
                            )
                        }
                    }
                },
                floatingActionButton = {
                    if (processState.currentState == ProcessStateConstants.IDLE) {
                        if (weHaveAllPermissions) {
                            ExtendedFloatingActionButton(onClick = {
                                homeScreenViewModel.runBackupSafely(
                                    showOnlyNew,
                                    context,
                                    currentLifeCycleOwner,
                                    uploadObserver
                                )
                            }) {
                                Icon(
                                    imageVector = Icons.Default.PlayArrow,
                                    contentDescription = ""
                                )
                                Text(text = "Start Backup")
                            }
                        } else {
                            ExtendedFloatingActionButton(onClick = {
                                permissionsNeeded.launchMultiplePermissionRequest()
                            }) {
                                Icon(
                                    imageVector = Icons.Default.AccountCircle,
                                    contentDescription = ""
                                )
                                Text(text = "Grant Permissions")
                            }
                        }
                    } else if (processState.currentState == ProcessStateConstants.UPLOADING) {
                        ExtendedFloatingActionButton(onClick = {
                            homeScreenViewModel.cancelBackup(context)
                        }) {
                            Icon(
                                imageVector = Icons.Default.Close,
                                contentDescription = ""
                            )
                            Text(text = "Cancel")
                        }
                    } else if (processState.currentState == ProcessStateConstants.DISCOVERING) {
                        ExtendedFloatingActionButton(onClick = {
                            homeScreenViewModel.cancelDiscovery(context)
                        }) {
                            Icon(
                                imageVector = Icons.Default.Close,
                                contentDescription = ""
                            )
                            Text(text = "Cancel")
                        }
                    }
                }
            )
        }
    )
}
