package com.exner.tools.immichuploader.ui

import android.Manifest
import android.app.Application
import android.content.ContentValues
import android.content.Context
import android.content.pm.PackageManager
import android.os.Build
import android.os.Environment
import android.provider.MediaStore
import android.util.Log
import android.widget.Toast
import androidx.core.content.ContextCompat
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.Observer
import androidx.lifecycle.viewModelScope
import androidx.work.Constraints
import androidx.work.Data
import androidx.work.ExistingWorkPolicy
import androidx.work.NetworkType
import androidx.work.OneTimeWorkRequest
import androidx.work.WorkInfo
import androidx.work.WorkManager
import com.exner.tools.immichuploader.data.MyRepository
import com.exner.tools.immichuploader.data.entities.MediaFile
import com.exner.tools.immichuploader.preferences.ExportablePreferences
import com.exner.tools.immichuploader.preferences.UserPreferencesRepository
import com.exner.tools.immichuploader.work.DiscoverMediaWorker
import com.exner.tools.immichuploader.work.UploadMediaAssetWorker
import com.exner.tools.immichuploader.work.UploadSingleMediaAssetWorker
import com.squareup.moshi.JsonAdapter
import com.squareup.moshi.Moshi
import com.squareup.moshi.adapter
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.firstOrNull
import kotlinx.coroutines.launch
import kotlinx.datetime.Clock
import kotlinx.datetime.TimeZone
import kotlinx.datetime.toLocalDateTime
import java.util.UUID
import javax.inject.Inject

private const val TAG = "HomeVM"

enum class ProcessStateConstants {
    IDLE,
    UPLOADING,
    PROBLEM,
    DISCOVERING,
    CANCELLING
}

data class ProcessState(
    val currentState: ProcessStateConstants = ProcessStateConstants.IDLE,
    val message: String? = null
)

@HiltViewModel
class HomeScreenViewModel @Inject constructor(
    application: Application,
    val userPreferencesRepository: UserPreferencesRepository,
    repository: MyRepository,
) : AndroidViewModel(application) {

    private val _processStateFlow = MutableStateFlow(ProcessState())
    val processStateFlow: StateFlow<ProcessState> = _processStateFlow.asStateFlow()

    val mediaFiles = repository.observerMediaFiles
    val mediaFilesNotBackedUp = repository.observeMediaFilesNotBackedUp
    val countMediaFiles = repository.countMediaFiles
    val countMediaFilesNotBackedUp = repository.countMediaFilesNotBackedUp
    val countFilesToDelete = repository.countFilesToDelete

    private var weHaveAccessToImages = false
    private var weHaveAccessToVideos = false
    private var weHaveAccessToLocations = false

    private var discoveryWorkerId: UUID? = null
    private var updateWorkerId: UUID? = null

    private val _uploadProgress: MutableStateFlow<Float> = MutableStateFlow(0.0f)
    val uploadProgress: StateFlow<Float> = _uploadProgress

    private var immichServerUrlText: String? = null
    private var immichApiKeyText: String? = null

    init {
        // cancel running workers, if there are any
        val workManager = WorkManager.getInstance(application.applicationContext)
        Log.d(TAG, "Cancelling 'old' workers...")
        workManager.cancelUniqueWork("ImmichUploaderDiscoveryWorker")
        workManager.cancelUniqueWork("ImmichUploaderUploadWorker")

        viewModelScope.launch {
            // check we have at least some permissions
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                // API level 33 or Android 13 or above
                weHaveAccessToImages = ContextCompat.checkSelfPermission(
                    application.applicationContext,
                    Manifest.permission.READ_MEDIA_IMAGES
                ) == PackageManager.PERMISSION_GRANTED
                weHaveAccessToVideos = ContextCompat.checkSelfPermission(
                    application.applicationContext,
                    Manifest.permission.READ_MEDIA_VIDEO
                ) == PackageManager.PERMISSION_GRANTED
            } else {
                // API levels 29 to 32 or Android 10 to 12
                weHaveAccessToImages = ContextCompat.checkSelfPermission(
                    application.applicationContext,
                    Manifest.permission.READ_EXTERNAL_STORAGE
                ) == PackageManager.PERMISSION_GRANTED
                weHaveAccessToVideos = weHaveAccessToImages
            }
            weHaveAccessToLocations = ContextCompat.checkSelfPermission(
                application.applicationContext,
                Manifest.permission.ACCESS_MEDIA_LOCATION
            ) == PackageManager.PERMISSION_GRANTED

            // immich server API settings from preference
            val immichServerUrl = userPreferencesRepository.immichServerUrl()
            immichServerUrlText = immichServerUrl.firstOrNull()
            val immichApiKey = userPreferencesRepository.immichApiKey()
            immichApiKeyText = immichApiKey.firstOrNull()
        }
    }

    fun loadImmichCredentials(callback: () -> Unit) {
        if (immichServerUrlText.isNullOrEmpty() || immichApiKeyText.isNullOrEmpty()) {
            // they are empty, let's try to load them
            viewModelScope.launch {
                immichServerUrlText =
                    userPreferencesRepository.immichServerUrl().firstOrNull()
                immichApiKeyText =
                    userPreferencesRepository.immichApiKey().firstOrNull()
                callback()
            }
        } else {
            callback()
        }
    }

    fun updateMedia(
        context: Context,
        lifecycleOwner: LifecycleOwner,
        observer: Observer<WorkInfo?>,
        force: Boolean = false
    ): UUID {
        val workManager = WorkManager.getInstance(context)
        val constraints = Constraints.Builder()
            .setRequiredNetworkType(NetworkType.CONNECTED)
            .setRequiresBatteryNotLow(true)
            .build()
        val data: Data = Data.Builder()
            .putBoolean(DiscoverMediaWorker.KEY_FORCE_SCAN, force)
            .build()
        val discoveryRequest = OneTimeWorkRequest.Builder(DiscoverMediaWorker::class)
            .setConstraints(constraints)
            .setInputData(data)
            .build()
        workManager.enqueueUniqueWork(
            "ImmichUploaderDiscoveryWorker",
            ExistingWorkPolicy.REPLACE, discoveryRequest
        )
        workManager.getWorkInfoByIdLiveData(discoveryRequest.id)
            .observe(lifecycleOwner, observer)

        discoveryWorkerId = discoveryRequest.id
        return discoveryRequest.id
    }

    fun cancelDiscovery(
        context: Context
    ) {
        val workManager = WorkManager.getInstance(context)
        if (discoveryWorkerId != null) {
            Log.d(TAG, "Cancelling discovery $discoveryWorkerId...")
            workManager.cancelWorkById(discoveryWorkerId!!)
        }
    }

    fun backupSingleAssetSafely(
        context: Context,
        mediaFile: MediaFile
    ) {
        loadImmichCredentials {
            backupSingleAsset(context, mediaFile)
        }
    }

    fun backupSingleAsset(
        context: Context,
        mediaFile: MediaFile,
    ) {
        if (!immichServerUrlText.isNullOrEmpty() && !immichApiKeyText.isNullOrEmpty()) {
            val workManager = WorkManager.getInstance(context)
            val constraints = Constraints.Builder()
                .setRequiredNetworkType(NetworkType.CONNECTED)
                .build()
            val data: Data = Data.Builder()
                .putString(UploadSingleMediaAssetWorker.KEY_MEDIA_URI_STRING, mediaFile.uri)
                .putString(
                    UploadSingleMediaAssetWorker.KEY_IMMICH_SERVER_URL_TEXT,
                    immichServerUrlText
                )
                .putString(UploadSingleMediaAssetWorker.KEY_IMMICH_API_KEY, immichApiKeyText)
                .build()
            val uploadAssetRequest =
                OneTimeWorkRequest.Builder(UploadSingleMediaAssetWorker::class)
                    .setConstraints(constraints)
                    .setInputData(data)
                    .build()
            workManager.enqueueUniqueWork(
                "ImmichUploaderSingleUploadWorker",
                ExistingWorkPolicy.APPEND,
                uploadAssetRequest
            )
            Log.d(TAG, "WM: $workManager")
        }
    }

    fun runBackupSafely(
        onlyNew: Boolean,
        context: Context,
        lifecycleOwner: LifecycleOwner,
        observer: Observer<WorkInfo?>
    ) {
        loadImmichCredentials {
            runBackup(
                onlyNew,
                context,
                lifecycleOwner,
                observer
            )
        }
    }

    fun runBackup(
        onlyNew: Boolean,
        context: Context,
        lifecycleOwner: LifecycleOwner,
        observer: Observer<WorkInfo?>
    ) {
        if (!immichServerUrlText.isNullOrEmpty() && !immichApiKeyText.isNullOrEmpty()) {
            val workManager = WorkManager.getInstance(context)
            val constraints = Constraints.Builder()
                .setRequiredNetworkType(NetworkType.CONNECTED)
                .setRequiresBatteryNotLow(true)
                .build()
            val data: Data = Data.Builder()
                .putBoolean(UploadMediaAssetWorker.KEY_ONLY_NEW_ASSETS, onlyNew)
                .putString(
                    UploadMediaAssetWorker.KEY_IMMICH_SERVER_URL_TEXT,
                    immichServerUrlText
                )
                .putString(UploadMediaAssetWorker.KEY_IMMICH_API_KEY, immichApiKeyText)
                .build()
            val uploadAssetRequest =
                OneTimeWorkRequest.Builder(UploadMediaAssetWorker::class)
                    .setConstraints(constraints)
                    .setInputData(data)
                    .build()

            Log.d(TAG, "Backup starting...")
            _processStateFlow.value = ProcessState(ProcessStateConstants.UPLOADING)

            workManager.enqueueUniqueWork(
                "ImmichUploaderUploadWorker",
                ExistingWorkPolicy.REPLACE,
                uploadAssetRequest
            )
            workManager.getWorkInfoByIdLiveData(uploadAssetRequest.id)
                .observe(lifecycleOwner, observer)
            updateWorkerId = uploadAssetRequest.id
        } else {
            _processStateFlow.value =
                ProcessState(ProcessStateConstants.PROBLEM, "Server URL must be defined!")
            Log.d(TAG, "U I S Url $immichServerUrlText")
            Log.d(TAG, "U I A Key $immichApiKeyText")
        }
    }

    fun cancelBackup(context: Context) {
        _processStateFlow.value = ProcessState(ProcessStateConstants.CANCELLING)
        val workManager = WorkManager.getInstance(context)
        if (updateWorkerId != null) {
            Log.d(TAG, "Cancelling discovery $updateWorkerId...")
            workManager.cancelWorkById(updateWorkerId!!)
        }
    }

    fun updateProcessState(processStateConstant: ProcessStateConstants) {
        _processStateFlow.value = ProcessState(processStateConstant)
    }

    fun setUploadProgress(progress: Float) {
        _uploadProgress.value = progress / 100f
    }

    @OptIn(ExperimentalStdlibApi::class)
    fun exportSettings(
        context: Context
    ) {
        viewModelScope.launch {
            try {
                val settingsToExport = ExportablePreferences(
                    immichServerUrl = immichServerUrlText,
                    immichApiKey = immichApiKeyText,
                    availablePseudoFolders = userPreferencesRepository.availablePseudoFolders()
                        .firstOrNull() ?: emptySet(),
                    deleteAssetsFromTheseAfterUpload = userPreferencesRepository.deleteMediaAfterSuccessfulUploadPerPseudoFolder()
                        .firstOrNull() ?: emptySet(),
                    latestMediaStoreVersion = userPreferencesRepository.latestMediaStoreVersion()
                        .firstOrNull() ?: "",
                    approveEachDeletion = userPreferencesRepository.approveEachDeletion()
                        .firstOrNull() == true
                )
                val moshi = Moshi.Builder()
                    .build()
                val jsonAdapter: JsonAdapter<ExportablePreferences> =
                    moshi.adapter<ExportablePreferences>()
                val settingsJsonString = jsonAdapter.toJson(settingsToExport)
                val now =
                    Clock.System.now()
                        .toLocalDateTime(TimeZone.Companion.currentSystemDefault())
                val timeComponentForExportFileName = now.toString()
                // now write it to the Downloads folder
                val contentValues = ContentValues().apply {
                    put(
                        MediaStore.MediaColumns.DISPLAY_NAME,
                        "immich-uploader-settings-$timeComponentForExportFileName"
                    )
                    put(MediaStore.MediaColumns.MIME_TYPE, "application/json")
                    put(MediaStore.MediaColumns.RELATIVE_PATH, Environment.DIRECTORY_DOWNLOADS)
                }
                val resolver = context.contentResolver
                val uri =
                    resolver.insert(MediaStore.Downloads.EXTERNAL_CONTENT_URI, contentValues)
                if (uri != null) {
                    resolver.openOutputStream(uri).use { stream ->
                        stream?.write(settingsJsonString.encodeToByteArray())
                    }
                }
                Toast.makeText(context, "Settings exported successfully", Toast.LENGTH_LONG)
                    .show()
            } catch (_: Exception) {
                Toast.makeText(context, "Problem exporting settings", Toast.LENGTH_LONG).show()
            }
        }
    }
}

