package com.exner.tools.immichuploader.ui

import androidx.compose.animation.AnimatedVisibility
import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.imePadding
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.KeyboardArrowDown
import androidx.compose.material.icons.filled.KeyboardArrowUp
import androidx.compose.material.icons.filled.MoreVert
import androidx.compose.material3.BottomAppBar
import androidx.compose.material3.DropdownMenu
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.HorizontalDivider
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.text.input.KeyboardCapitalization
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import com.ramcosta.composedestinations.annotation.Destination
import com.ramcosta.composedestinations.annotation.RootGraph
import com.ramcosta.composedestinations.navigation.DestinationsNavigator
import io.github.vinceglb.filekit.dialogs.FileKitMode
import io.github.vinceglb.filekit.dialogs.FileKitType
import io.github.vinceglb.filekit.dialogs.compose.rememberFilePickerLauncher

@OptIn(ExperimentalMaterial3Api::class, ExperimentalFoundationApi::class)
@Destination<RootGraph>
@Composable
fun Settings(
    settingsViewModel: SettingsViewModel = hiltViewModel(),
    destinationsNavigator: DestinationsNavigator
) {
    val immichServerUrlText by settingsViewModel.immichServerUrl.collectAsStateWithLifecycle(
        initialValue = null
    )
    val immichApiKey by settingsViewModel.immichApiKey.collectAsStateWithLifecycle(
        initialValue = null
    )

    val availablePseudoFolders by settingsViewModel.availablePseudoFolders.collectAsStateWithLifecycle(
        initialValue = emptySet()
    )

    val deleteMediaAfterSuccessfulUploadPerPseudoFolder by settingsViewModel.deleteMediaAfterSuccessfulUploadPerPseudoFolder.collectAsStateWithLifecycle(
        initialValue = emptySet()
    )

    val approveEachDeletion by settingsViewModel.approveEachDeletion.collectAsStateWithLifecycle(
        initialValue = true
    )

    val settingsVisible by settingsViewModel.areImmichServerSettingsVisible.collectAsStateWithLifecycle(
        availablePseudoFolders.isNotEmpty()
    )

    var displayMainMenu by remember { mutableStateOf(false) }
    val context = LocalContext.current

    val launcher = rememberFilePickerLauncher(
        type = FileKitType.File(extensions = listOf("json")),
        mode = FileKitMode.Single,
        title = "Select a file to import"
    ) { file ->
        if (file != null) {
            settingsViewModel.importSettings(context, file)
        }
    }

    Scaffold(
        topBar = {
            TopAppBar(
                title = {
                    Text(text = "Immich Uploader")
                },
                navigationIcon = {
                    IconButton(onClick = {
                        destinationsNavigator.navigateUp()
                    }) {
                        Icon(
                            imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                            contentDescription = "Back"
                        )
                    }
                },
                actions = {
                    IconButton(
                        onClick = {
                            displayMainMenu = !displayMainMenu
                        }
                    ) {
                        Icon(
                            imageVector = Icons.Filled.MoreVert,
                            contentDescription = "Menu"
                        )
                    }
                    DropdownMenu(
                        expanded = displayMainMenu,
                        onDismissRequest = { displayMainMenu = false }
                    ) {
                        DropdownMenuItem(
                            enabled = true,
                            text = { Text(text = "Import settings") },
                            onClick = {
                                displayMainMenu = false
                                launcher.launch()
                            }
                        )
//                        HorizontalDivider()
//                        DropdownMenuItem(
//                            enabled = destination != AboutDestination,
//                            text = { Text(text = "About Meditation Timer") },
//                            onClick = {
//                                displayMainMenu = false
//                                destinationsNavigator.navigate(AboutDestination)
//                            }
//                        )
                    }
                }
            )
        },
        content = { innerPadding ->
            LazyColumn(
                modifier = Modifier
                    .imePadding()
                    .padding(innerPadding)
                    .padding(8.dp)
                    .fillMaxSize()
            ) {
                //
                // sections for the immich server
                //
                stickyHeader {
                    Box(
                        modifier = Modifier
                            .padding(0.dp, 8.dp, 0.dp, 0.dp)
                            .fillMaxWidth()
                            .background(MaterialTheme.colorScheme.primaryContainer)
                            .clickable {
                                settingsViewModel.setImmichServerSettingsVisible(!settingsVisible)
                            },
                    ) {
                        Text(
                            text = "Immich Server Settings",
                            color = MaterialTheme.colorScheme.onPrimaryContainer
                        )
                        if (settingsVisible) {
                            Icon(
                                imageVector = Icons.Default.KeyboardArrowUp,
                                contentDescription = "Hide Settings",
                                modifier = Modifier.align(Alignment.CenterEnd)
                            )
                        } else {
                            Icon(
                                imageVector = Icons.Default.KeyboardArrowDown,
                                contentDescription = "Show Settings",
                                modifier = Modifier.align(Alignment.CenterEnd)
                            )
                        }
                    }
                }
                item {
                    AnimatedVisibility(
                        visible = settingsVisible
                    ) {
                        OutlinedTextField(
                            value = immichServerUrlText ?: "",
                            onValueChange = {
                                settingsViewModel.setImmichServerUrl(it)
                            },
                            label = {
                                Text(text = "Immich Server URL")
                            },
                            placeholder = {
                                Text(text = "http(s)://immich.server:2283")
                            },
                            modifier = Modifier
                                .fillMaxWidth()
                                .padding(0.dp, 4.dp),
                            keyboardOptions = KeyboardOptions(
                                capitalization = KeyboardCapitalization.None
                            )
                        )
                    }
                    AnimatedVisibility(
                        visible = settingsVisible
                    ) {
                        OutlinedTextField(
                            value = immichApiKey ?: "",
                            onValueChange = {
                                settingsViewModel.setImmichApiKey(it)
                            },
                            label = {
                                Text(text = "Immich API Key")
                            },
                            placeholder = {
                                Text(text = "de21f88bb3...")
                            },
                            modifier = Modifier
                                .fillMaxWidth()
                                .padding(0.dp, 4.dp),
                            keyboardOptions = KeyboardOptions(
                                capitalization = KeyboardCapitalization.None
                            )
                        )
                    }
                    HorizontalDivider()
                }

                //
                // settings about which folders to scan
                //
                stickyHeader {
                    if (availablePseudoFolders.isNotEmpty()) {
                        SectionHeader(
                            text = "Upload Settings (per \"Category\")"
                        )
                    }
                }
                item {
                    if (availablePseudoFolders.isNotEmpty()) {
                        TextAndSwitch(
                            text = "Approve removal of each file individually",
                            checked = approveEachDeletion
                        ) {
                            settingsViewModel.setApproveEachDeletion(it)
                        }
                    }
                }
                item {
                    if (availablePseudoFolders.isNotEmpty()) {

                        Text(text = "Mark media in the following categories for deletion after upload:")
                    }
                }
                availablePseudoFolders.sorted().forEach { folderName ->
                    item {
                        val indent = folderName.count { it == '/' }
                        TextAndSwitch(
                            indent = indent,
                            text = folderName,
                            checked = deleteMediaAfterSuccessfulUploadPerPseudoFolder.contains(
                                folderName
                            )
                        ) { deleteIt ->
                            settingsViewModel.setDeleteMediaAfterSuccessfulUploadForPseudoFolder(
                                folderName, deleteIt
                            )
                        }
                    }
                }
            }
        },
        bottomBar = {
            BottomAppBar(
                actions = {
                    IconButton(
                        onClick = {
                            destinationsNavigator.navigateUp()
                        }
                    ) {
                        Icon(
                            imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                            contentDescription = "Back"
                        )
                    }
                },
                floatingActionButton = {}
            )
        }
    )
}
