package com.exner.tools.immichuploader.ui

import android.content.Context
import android.util.Log
import android.widget.Toast
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.exner.tools.immichuploader.preferences.ExportablePreferences
import com.exner.tools.immichuploader.preferences.UserPreferencesRepository
import com.squareup.moshi.JsonAdapter
import com.squareup.moshi.Moshi
import com.squareup.moshi.adapter
import dagger.hilt.android.lifecycle.HiltViewModel
import io.github.vinceglb.filekit.PlatformFile
import io.github.vinceglb.filekit.readString
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.firstOrNull
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class SettingsViewModel @Inject constructor(
    private val userPreferencesRepository: UserPreferencesRepository
) : ViewModel() {

    private val _areImmichServerSettingsVisible: MutableStateFlow<Boolean> = MutableStateFlow(false)
    val areImmichServerSettingsVisible: StateFlow<Boolean> = _areImmichServerSettingsVisible
    fun setImmichServerSettingsVisible(visible: Boolean) {
        _areImmichServerSettingsVisible.value = visible
    }

    val immichServerUrl = userPreferencesRepository.immichServerUrl()
    fun setImmichServerUrl(newUrl: String?) {
        Log.d("SettingsVM", "Setting I S URL |$newUrl|")
        viewModelScope.launch {
            userPreferencesRepository.setImmichServerUrl(newUrl)
        }
    }

    val immichApiKey = userPreferencesRepository.immichApiKey()
    fun setImmichApiKey(newKey: String?) {
        Log.d("SettingsVM", "Setting I A Key |$newKey|")
        viewModelScope.launch {
            userPreferencesRepository.setImmichApiKey(newKey)
        }
    }

    val availablePseudoFolders = userPreferencesRepository.availablePseudoFolders()

    val deleteMediaAfterSuccessfulUploadPerPseudoFolder =
        userPreferencesRepository.deleteMediaAfterSuccessfulUploadPerPseudoFolder()

    fun setDeleteMediaAfterSuccessfulUploadForPseudoFolder(
        folder: String,
        deleteAU: Boolean
    ) {
        viewModelScope.launch {
            val temp = userPreferencesRepository.deleteMediaAfterSuccessfulUploadPerPseudoFolder()
                .firstOrNull() ?: setOf()
            val mutableTemp = temp.toMutableSet()
            if (deleteAU) {
                mutableTemp.add(folder)
            } else {
                mutableTemp.remove(folder)
            }
            userPreferencesRepository.setDeleteMediaAfterSuccessfulUploadPerPseudoFolder(mutableTemp)
        }
    }

    val approveEachDeletion = userPreferencesRepository.approveEachDeletion()
    fun setApproveEachDeletion(approve: Boolean) {
        viewModelScope.launch {
            userPreferencesRepository.setApproveEachDeletion(approve)
        }
    }

    @OptIn(ExperimentalStdlibApi::class)
    fun importSettings(context: Context, file: PlatformFile) {
        viewModelScope.launch {
            try {
                val moshi = Moshi.Builder().build()
                val jsonAdapter: JsonAdapter<ExportablePreferences> =
                    moshi.adapter<ExportablePreferences>()
                val fileContent = file.readString()
                val settingsToImport = jsonAdapter.fromJson(fileContent)
                if (settingsToImport != null) {
                    setImmichServerUrl(settingsToImport.immichServerUrl)
                    setImmichApiKey(settingsToImport.immichApiKey)
                    userPreferencesRepository.setAvailablePseudoFolders(settingsToImport.availablePseudoFolders)
                    userPreferencesRepository.setDeleteMediaAfterSuccessfulUploadPerPseudoFolder(
                        settingsToImport.deleteAssetsFromTheseAfterUpload
                    )
                    userPreferencesRepository.setLatestMediaStoreVersion(settingsToImport.latestMediaStoreVersion)
                    userPreferencesRepository.setApproveEachDeletion(settingsToImport.approveEachDeletion != false)
                }
                Toast.makeText(context, "Import of settings succeeded", Toast.LENGTH_LONG).show()
            } catch (e: Exception) {
                Toast.makeText(context, "Import of settings failed", Toast.LENGTH_LONG).show()
                Log.d("SVM", "Import failed: ${e.message}")
            }
        }
    }
}
