package com.exner.tools.immichuploader.work

import android.content.ContentUris
import android.content.Context
import android.net.Uri
import android.provider.MediaStore
import android.util.Log
import androidx.hilt.work.HiltWorker
import androidx.work.CoroutineWorker
import androidx.work.Data
import androidx.work.WorkerParameters
import androidx.work.workDataOf
import com.exner.tools.immichuploader.data.MyRepository
import com.exner.tools.immichuploader.data.entities.MediaFile
import com.exner.tools.immichuploader.preferences.UserPreferencesRepository
import dagger.assisted.Assisted
import dagger.assisted.AssistedInject
import kotlinx.coroutines.flow.firstOrNull
import kotlinx.datetime.DateTimeUnit
import kotlinx.datetime.Instant
import kotlinx.datetime.TimeZone
import kotlinx.datetime.until

//
// Worker that goes through all assets in the MediaStore
//
// found assets will be added to or updated in the database
//

@HiltWorker
class DiscoverMediaWorker @AssistedInject constructor(
    @Assisted val context: Context,
    @Assisted workerParams: WorkerParameters,
    private val userPreferencesRepository: UserPreferencesRepository,
    private val repository: MyRepository
) : CoroutineWorker(context, workerParams) {

    override suspend fun doWork(): Result {
        // get input data
        val forceScan: Boolean = inputData.getBoolean(KEY_FORCE_SCAN, false)

        val latestMediaStoreVersion =
            userPreferencesRepository.latestMediaStoreVersion().firstOrNull() ?: ""
        val mediaFilesInDatabase = repository.getAllMediaFiles()

        var listOfPseudoFolders = userPreferencesRepository.availablePseudoFolders().firstOrNull()?.toMutableSet() ?: mutableSetOf()

        // do stuff
        if (mediaFilesInDatabase.isEmpty() || forceScan || latestMediaStoreVersion != MediaStore.getVersion(
                context
            )
        ) {
            if (!isStopped) {
                // reset list of discovered pseudo folders
                listOfPseudoFolders = mutableSetOf()
            }
            // go
            if (!isStopped) {
                setProgress(workDataOf(PROGRESS to 0))
            }
            // scan everything
            if (!isStopped) {
                listOfPseudoFolders.addAll(discoverImages())
            }
            if (!isStopped) {
                setProgress(workDataOf(PROGRESS to 50))
            }
            if (!isStopped) {
                listOfPseudoFolders.addAll(discoverVideos())
            }
            if (!isStopped) {
                // record MediaStore version
                userPreferencesRepository.setLatestMediaStoreVersion(MediaStore.getVersion(context))
                setProgress(workDataOf(PROGRESS to 100))
            }

            if (!isStopped) {
                // record all pseudo folder
                userPreferencesRepository.setAvailablePseudoFolders(listOfPseudoFolders)
            }
        }

        if (isStopped) {
            return Result.success()
        }

        // count assets
        val assetsDiscovered = repository.countMediaFiles.firstOrNull()

        // make return data
        val outputData = Data
            .Builder()
            .putInt(KEY_SCAN_RESULT_ASSETS_COUNT, assetsDiscovered ?: 0)
            .build()
        return Result.success(outputData)
    }

    private suspend fun discoverVideos(): Set<String> {
        //
        // discover videos
        //
        var listOfPseudoFolders: MutableSet<String> = mutableSetOf()

        val videoProjection = arrayOf(
            MediaStore.Video.Media._ID,
            MediaStore.Video.Media.DATE_TAKEN,
            MediaStore.Video.Media.DATE_MODIFIED,
            MediaStore.Video.Media.DISPLAY_NAME,
            MediaStore.MediaColumns.RELATIVE_PATH,
        )
        val videoSortOrder = "${MediaStore.Video.Media.DATE_ADDED} DESC"
        val videoCursor = context.contentResolver.query(
            MediaStore.Video.Media.EXTERNAL_CONTENT_URI,
            videoProjection,
            null,
            null,
            videoSortOrder
        )
        videoCursor.use {
            if (videoCursor != null) {
                val idColumn = videoCursor.getColumnIndexOrThrow(MediaStore.Video.Media._ID)
                val dateTakenColumn =
                    videoCursor.getColumnIndexOrThrow(MediaStore.Video.Media.DATE_TAKEN)
                val dateUpdatedColumn =
                    videoCursor.getColumnIndexOrThrow(MediaStore.Video.Media.DATE_MODIFIED)
                val displayNameColumn =
                    videoCursor.getColumnIndexOrThrow(MediaStore.Video.Media.DISPLAY_NAME)
                val relativePathColumn =
                    videoCursor.getColumnIndexOrThrow(MediaStore.MediaColumns.RELATIVE_PATH)
                while (videoCursor.moveToNext()) {
                    if (!isStopped) {
                        val id = videoCursor.getLong(idColumn)
                        val contentUri = ContentUris.withAppendedId(
                            MediaStore.Video.Media.EXTERNAL_CONTENT_URI,
                            id
                        )
                        val dateTaken = videoCursor.getLong(dateTakenColumn)
                        val dateTakenInstant = Instant.fromEpochMilliseconds(dateTaken)
                        val dateUpdated = videoCursor.getLong(dateUpdatedColumn)
                        val dateUpdatedInstant = Instant.fromEpochMilliseconds(dateUpdated)
                        val assetName = videoCursor.getString(displayNameColumn)
                        val relativePath = videoCursor.getString(relativePathColumn)
                        listOfPseudoFolders.add(relativePath.dropLast(1))
                        // also add to database
                        addMediaFileToDatabase(
                            contentUri,
                            assetName,
                            dateTakenInstant,
                            dateUpdatedInstant,
                            id,
                            relativePath.dropLast(1)
                        )
                    }
                }
                videoCursor.close()
            } else {
                Log.d("AddViewModel", "videoCursor is null!")
            }
        }

        return listOfPseudoFolders
    }

    private suspend fun discoverImages(): Set<String> {
        //
        // discover images
        //
        var listOfPseudoFolders: MutableSet<String> = mutableSetOf()

        val imageProjection = arrayOf(
            MediaStore.Images.Media._ID,
            MediaStore.Images.Media.DATE_TAKEN,
            MediaStore.Images.Media.DATE_MODIFIED,
            MediaStore.Images.Media.DISPLAY_NAME,
            MediaStore.MediaColumns.RELATIVE_PATH,
        )
        val imageSortOrder = "${MediaStore.Images.Media.DATE_ADDED} DESC"
        val cursor = context.contentResolver.query(
            MediaStore.Images.Media.EXTERNAL_CONTENT_URI,
            imageProjection,
            null,
            null,
            imageSortOrder
        )
        cursor.use {
            if (cursor != null) {
                val idColumn = cursor.getColumnIndexOrThrow(MediaStore.Images.Media._ID)
                val dateTakenColumn =
                    cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATE_TAKEN)
                val dateUpdatedColumn =
                    cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATE_MODIFIED)
                val displayNameColumn =
                    cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DISPLAY_NAME)
                val relativePathColumn =
                    cursor.getColumnIndexOrThrow(MediaStore.MediaColumns.RELATIVE_PATH)
                while (cursor.moveToNext()) {
                    if (!isStopped) {
                        val id = cursor.getLong(idColumn)
                        val contentUri = ContentUris.withAppendedId(
                            MediaStore.Images.Media.EXTERNAL_CONTENT_URI,
                            id
                        )
                        val dateTaken = cursor.getLong(dateTakenColumn)
                        val dateTakenInstant = Instant.fromEpochMilliseconds(dateTaken)
                        val dateUpdated = cursor.getLong(dateUpdatedColumn)
                        val dateUpdatedInstant = Instant.fromEpochMilliseconds(dateUpdated)
                        val assetName = cursor.getString(displayNameColumn)
                        val relativePath = cursor.getString(relativePathColumn)
                        listOfPseudoFolders.add(relativePath.dropLast(1))
                        // also add to database
                        addMediaFileToDatabase(
                            contentUri,
                            assetName,
                            dateTakenInstant,
                            dateUpdatedInstant,
                            id,
                            relativePath.dropLast(1)
                        )
                    }
                }
                cursor.close()
            } else {
                Log.d("AddViewModel", "Cursor is null!")
            }
        }

        return listOfPseudoFolders
    }

    private suspend fun addMediaFileToDatabase(
        contentUri: Uri,
        name: String,
        dateTakenInstant: Instant,
        dateUpdatedInstant: Instant,
        id: Long,
        pseudoFolder: String
    ) {
        // check whether this needs to be updated or inserted
        val mediaFile = repository.getMediaFile(contentUri.toString())
        if (mediaFile != null) {
            // file exists in DB and will be updated
            var newMediaFile = if (mediaFile.createdDate == null) {
                mediaFile.copy(
                    createdDate = dateTakenInstant,
                    pseudoFolder = pseudoFolder
                )
            } else {
                mediaFile.copy(
                    pseudoFolder = pseudoFolder
                )
            }
            if (newMediaFile.lastBackup != null) {
                if (newMediaFile.lastBackup.until(
                        dateUpdatedInstant, DateTimeUnit.SECOND,
                        TimeZone.currentSystemDefault()
                    ) > 0
                ) {
                    // last updated happened before last backup
                    // so we'll remove the lastBackup date to make it look new
                    newMediaFile = newMediaFile.copy(
                        lastBackup = null
                    )
                }
            }
            repository.updateMediaFile(newMediaFile)
        } else {
            // file does not yet exist and will be inserted
            val newMediaFile = MediaFile(
                uri = contentUri.toString(),
                name = name,
                deviceAssetId = id,
                backupStatus = null,
                lastBackup = null,
                immichAssetId = null,
                createdDate = dateTakenInstant,
                updatedDate = dateUpdatedInstant,
                pseudoFolder = pseudoFolder
            )
            repository.insertMediaFile(newMediaFile)
        }
    }

    companion object {
        const val PROGRESS = "progress"
        const val KEY_FORCE_SCAN = "key_force_scan"
        const val KEY_SCAN_RESULT_ASSETS_COUNT = "key_scan_result_new_assets_count"
    }
}
