package com.exner.tools.immichuploader.work

import android.util.Log
import androidx.work.CoroutineWorker
import androidx.work.workDataOf
import com.exner.tools.immichuploader.data.MyRepository
import com.exner.tools.immichuploader.data.entities.FileDeletionCache
import com.exner.tools.immichuploader.data.entities.MediaFile
import com.exner.tools.immichuploader.network.ImmichApiManager
import com.exner.tools.immichuploader.network.models.ImmichUploadResult
import kotlinx.coroutines.CancellationException
import kotlinx.datetime.Clock

private const val TAG = "UA"

suspend fun uploadAsset(
    repository: MyRepository,
    counter: Int,
    numberOfAssetsToUpload: Int,
    mediaFile: MediaFile,
    immichApiManager: ImmichApiManager,
    immichServerUrlText: String,
    immichApiKey: String,
    deviceId: String,
    imageCategoriesThatShouldBeDeletedAfterSuccessfulUpload: Set<String>?,
    coroutineWorker: CoroutineWorker?
) {
    val PROGRESS = "Progress"
    val CURRENT_ASSET = "Asset"
    val STATUS = "Status"

    try {
        // notify progress
        val uploadProgress: Float = (100.0f * counter) / numberOfAssetsToUpload
        coroutineWorker?.setProgress(
            workDataOf(
                PROGRESS to uploadProgress,
                CURRENT_ASSET to mediaFile.name,
                STATUS to "uploading"
            )
        )
        Log.d(TAG, "U $counter of $numberOfAssetsToUpload - $uploadProgress...")
        // upload it!
        val immichUploadResult = immichApiManager.uploadAsset(
            url = "${immichServerUrlText}/api/assets",
            apiKey = immichApiKey,
            mediaFile = mediaFile,
            deviceId = deviceId,
        )
        // update the database with the immichId and upload status
        if (immichUploadResult is ImmichUploadResult.ImmichUploadSuccess && !immichUploadResult.id.isNullOrEmpty() && !immichUploadResult.status.isNullOrEmpty()) {
            coroutineWorker?.setProgress(
                workDataOf(
                    PROGRESS to uploadProgress,
                    CURRENT_ASSET to mediaFile.name,
                    STATUS to "done"
                )
            )

            val updatedMediaFile = mediaFile.copy(
                lastBackup = Clock.System.now(),
                immichAssetId = immichUploadResult.id,
                backupStatus = immichUploadResult.status
            )
            repository.updateMediaFile(updatedMediaFile)
            Log.d(TAG, "U uploaded $updatedMediaFile")

            // should this be deleted?
            val setOfCategoriesToBeDeleted =
                imageCategoriesThatShouldBeDeletedAfterSuccessfulUpload
            if (!setOfCategoriesToBeDeleted.isNullOrEmpty()) {
                val shouldBeDeleted =
                    setOfCategoriesToBeDeleted.contains(mediaFile.pseudoFolder)
                if (shouldBeDeleted) {
                    Log.d(
                        TAG,
                        "U Marking file for deletion after successful upload $mediaFile"
                    )
                    val fileToDelete = FileDeletionCache(
                        uriString = mediaFile.uri,
                        fileName = mediaFile.name,
                        delete = true
                    )
                    repository.insertFileToDelete(fileToDelete)
                }
            }
        } else {
            coroutineWorker?.setProgress(
                workDataOf(
                    PROGRESS to uploadProgress,
                    CURRENT_ASSET to mediaFile.name,
                    STATUS to "failed"
                )
            )
            Log.i(TAG, "U Something went wrong during upload of $mediaFile!")
            Log.i(TAG, "     $immichUploadResult")
        }
    } catch (e: Exception) {
        if (e is CancellationException) {
            Log.d(TAG, "U cancelled.")
        } else {
            Log.e(TAG, "U Exception: $e")
        }
    }
}